@@ -16,10 +16,10 @@ from typing import Optional, Tuple
 from distutils.util import strtobool
 
 from importlib.metadata import version
-from emhass.retrieve_hass import retrieve_hass
-from emhass.forecast import forecast
-from emhass.machine_learning_forecaster import mlforecaster
-from emhass.optimization import optimization
+from emhass.retrieve_hass import RetrieveHass
+from emhass.forecast import Forecast
+from emhass.machine_learning_forecaster import MLForecaster
+from emhass.optimization import Optimization
 from emhass import utils
 
 
@@ -58,12 +58,12 @@ def set_input_data_dict(config_path: pathlib.Path, base_path: str, costfun: str,
         runtimeparams, params, retrieve_hass_conf, 
         optim_conf, plant_conf, set_type, logger)
     # Define main objects
-    rh = retrieve_hass(retrieve_hass_conf['hass_url'], retrieve_hass_conf['long_lived_token'], 
+    rh = RetrieveHass(retrieve_hass_conf['hass_url'], retrieve_hass_conf['long_lived_token'], 
                        retrieve_hass_conf['freq'], retrieve_hass_conf['time_zone'], 
                        params, base_path, logger, get_data_from_file=get_data_from_file)
-    fcst = forecast(retrieve_hass_conf, optim_conf, plant_conf,
+    fcst = Forecast(retrieve_hass_conf, optim_conf, plant_conf,
                     params, base_path, logger, get_data_from_file=get_data_from_file)
-    opt = optimization(retrieve_hass_conf, optim_conf, plant_conf, 
+    opt = Optimization(retrieve_hass_conf, optim_conf, plant_conf, 
                        fcst.var_load_cost, fcst.var_prod_price, 
                        costfun, base_path, logger)
     # Perform setup based on type of action
@@ -75,12 +75,14 @@ def set_input_data_dict(config_path: pathlib.Path, base_path: str, costfun: str,
         else:
             days_list = utils.get_days_list(retrieve_hass_conf['days_to_retrieve'])
             var_list = [retrieve_hass_conf['var_load'], retrieve_hass_conf['var_PV']]
-            rh.get_data(days_list, var_list,
-                        minimal_response=False, significant_changes_only=False)
-        rh.prepare_data(retrieve_hass_conf['var_load'], load_negative = retrieve_hass_conf['load_negative'],
+            if not rh.get_data(days_list, var_list,
+                        minimal_response=False, significant_changes_only=False):
+                return False 
+        if not rh.prepare_data(retrieve_hass_conf['var_load'], load_negative = retrieve_hass_conf['load_negative'],
                         set_zero_min = retrieve_hass_conf['set_zero_min'], 
                         var_replace_zero = retrieve_hass_conf['var_replace_zero'], 
-                        var_interp = retrieve_hass_conf['var_interp'])
+                        var_interp = retrieve_hass_conf['var_interp']):
+            return False
         df_input_data = rh.df_final.copy()
         # What we don't need for this type of action
         P_PV_forecast, P_load_forecast, df_input_data_dayahead = None, None, None
@@ -89,6 +91,9 @@ def set_input_data_dict(config_path: pathlib.Path, base_path: str, costfun: str,
         df_weather = fcst.get_weather_forecast(method=optim_conf['weather_forecast_method'])
         P_PV_forecast = fcst.get_power_from_weather(df_weather)
         P_load_forecast = fcst.get_load_forecast(method=optim_conf['load_forecast_method'])
+        if isinstance(P_load_forecast,bool) and not P_load_forecast:
+            logger.error("Unable to get sensor_power_photovoltaics or sensor_power_load_no_var_loads")
+            return False
         df_input_data_dayahead = pd.DataFrame(np.transpose(np.vstack([P_PV_forecast.values,P_load_forecast.values])),
                                               index=P_PV_forecast.index,
                                               columns=['P_PV_forecast', 'P_load_forecast'])
@@ -107,12 +112,14 @@ def set_input_data_dict(config_path: pathlib.Path, base_path: str, costfun: str,
         else:
             days_list = utils.get_days_list(1)
             var_list = [retrieve_hass_conf['var_load'], retrieve_hass_conf['var_PV']]
-            rh.get_data(days_list, var_list,
-                        minimal_response=False, significant_changes_only=False)
-        rh.prepare_data(retrieve_hass_conf['var_load'], load_negative = retrieve_hass_conf['load_negative'],
+            if not rh.get_data(days_list, var_list,
+                        minimal_response=False, significant_changes_only=False):
+                return False
+        if not rh.prepare_data(retrieve_hass_conf['var_load'], load_negative = retrieve_hass_conf['load_negative'],
                         set_zero_min = retrieve_hass_conf['set_zero_min'], 
                         var_replace_zero = retrieve_hass_conf['var_replace_zero'], 
-                        var_interp = retrieve_hass_conf['var_interp'])
+                        var_interp = retrieve_hass_conf['var_interp']):
+            return False
         df_input_data = rh.df_final.copy()
         # Get PV and load forecasts
         df_weather = fcst.get_weather_forecast(method=optim_conf['weather_forecast_method'])
@@ -143,7 +150,8 @@ def set_input_data_dict(config_path: pathlib.Path, base_path: str, costfun: str,
         else:
             days_list = utils.get_days_list(days_to_retrieve)
             var_list = [var_model]
-            rh.get_data(days_list, var_list)
+            if not rh.get_data(days_list, var_list):
+                return False
             df_input_data = rh.df_final.copy()
     elif set_type == "publish-data":
         df_input_data, df_input_data_dayahead = None, None
@@ -272,9 +280,11 @@ def naive_mpc_optim(input_data_dict: dict, logger: logging.Logger,
     soc_init = input_data_dict['params']['passed_data']['soc_init']
     soc_final = input_data_dict['params']['passed_data']['soc_final']
     def_total_hours = input_data_dict['params']['passed_data']['def_total_hours']
+    def_start_timestep = input_data_dict['params']['passed_data']['def_start_timestep']
+    def_end_timestep = input_data_dict['params']['passed_data']['def_end_timestep']
     opt_res_naive_mpc = input_data_dict['opt'].perform_naive_mpc_optim(
         df_input_data_dayahead, input_data_dict['P_PV_forecast'], input_data_dict['P_load_forecast'],
-        prediction_horizon, soc_init, soc_final, def_total_hours)
+        prediction_horizon, soc_init, soc_final, def_total_hours, def_start_timestep, def_end_timestep)
     # Save CSV file for publish_data
     if save_data_to_file:
         today = datetime.now(timezone.utc).replace(hour=0, minute=0, second=0, microsecond=0)
@@ -286,7 +296,7 @@ def naive_mpc_optim(input_data_dict: dict, logger: logging.Logger,
     return opt_res_naive_mpc
 
 def forecast_model_fit(input_data_dict: dict, logger: logging.Logger,
-    debug: Optional[bool] = False) -> Tuple[pd.DataFrame, pd.DataFrame, mlforecaster]:
+    debug: Optional[bool] = False) -> Tuple[pd.DataFrame, pd.DataFrame, MLForecaster]:
     """Perform a forecast model fit from training data retrieved from Home Assistant.
 
     :param input_data_dict: A dictionnary with multiple data used by the action functions
@@ -307,7 +317,7 @@ def forecast_model_fit(input_data_dict: dict, logger: logging.Logger,
     perform_backtest = input_data_dict['params']['passed_data']['perform_backtest']
     root = input_data_dict['root']
     # The ML forecaster object
-    mlf = mlforecaster(data, model_type, var_model, sklearn_model, num_lags, root, logger)
+    mlf = MLForecaster(data, model_type, var_model, sklearn_model, num_lags, root, logger)
     # Fit the ML model
     df_pred, df_pred_backtest = mlf.fit(split_date_delta=split_date_delta, 
                                         perform_backtest=perform_backtest)
@@ -320,7 +330,7 @@ def forecast_model_fit(input_data_dict: dict, logger: logging.Logger,
 
 def forecast_model_predict(input_data_dict: dict, logger: logging.Logger,
     use_last_window: Optional[bool] = True, debug: Optional[bool] = False,
-    mlf: Optional[mlforecaster] = None) -> pd.DataFrame:
+    mlf: Optional[MLForecaster] = None) -> pd.DataFrame:
     r"""Perform a forecast model predict using a previously trained skforecast model.
 
     :param input_data_dict: A dictionnary with multiple data used by the action functions
@@ -365,7 +375,7 @@ def forecast_model_predict(input_data_dict: dict, logger: logging.Logger,
     model_predict_unit_of_measurement = input_data_dict['params']['passed_data']['model_predict_unit_of_measurement']
     model_predict_friendly_name = input_data_dict['params']['passed_data']['model_predict_friendly_name']
     publish_prefix = input_data_dict['params']['passed_data']['publish_prefix']
-    if model_predict_publish:
+    if model_predict_publish is True:
         # Estimate the current index
         now_precise = datetime.now(input_data_dict['retrieve_hass_conf']['time_zone']).replace(second=0, microsecond=0)
         if input_data_dict['retrieve_hass_conf']['method_ts_round'] == 'nearest':
@@ -386,8 +396,8 @@ def forecast_model_predict(input_data_dict: dict, logger: logging.Logger,
     return predictions
 
 def forecast_model_tune(input_data_dict: dict, logger: logging.Logger,
-    debug: Optional[bool] = False, mlf: Optional[mlforecaster] = None
-    ) -> Tuple[pd.DataFrame, mlforecaster]:
+    debug: Optional[bool] = False, mlf: Optional[MLForecaster] = None
+    ) -> Tuple[pd.DataFrame, MLForecaster]:
     """Tune a forecast model hyperparameters using bayesian optimization.
 
     :param input_data_dict: A dictionnary with multiple data used by the action functions
@@ -413,7 +423,7 @@ def forecast_model_tune(input_data_dict: dict, logger: logging.Logger,
                 mlf = pickle.load(inp)
         else:
             logger.error("The ML forecaster file was not found, please run a model fit method before this tune method")
-            return
+            return None, None
     # Tune the model
     df_pred_optim = mlf.tune(debug=debug)
     # Save model
@@ -538,6 +548,9 @@ def publish_data(input_data_dict: dict, logger: logging.Logger,
                                     publish_prefix = publish_prefix)
     # Publish the optimization status
     custom_cost_fun_id = params['passed_data']['custom_optim_status_id']
+    if "optim_status" not in opt_res_latest:
+        opt_res_latest["optim_status"] = 'Optimal'
+        logger.warning("no optim_status in opt_res_latest, run an optimization task first")
     input_data_dict['rh'].post_data(opt_res_latest['optim_status'], idx_closest, 
                                     custom_cost_fun_id["entity_id"], 
                                     custom_cost_fun_id["unit_of_measurement"],
