@@ -13,7 +13,7 @@ from requests import get, post
 from emhass.utils import set_df_index_freq
 
 
-class retrieve_hass:
+class RetrieveHass:
     r"""
     Retrieve data from Home Assistant using the restful API.
     
@@ -34,7 +34,7 @@ class retrieve_hass:
                  time_zone: datetime.timezone, params: str, base_path: str, logger: logging.Logger,
                  get_data_from_file: Optional[bool] = False) -> None:
         """
-        Define constructor for retrieve_hass class.
+        Define constructor for RetrieveHass class.
         
         :param hass_url: The URL of the Home Assistant instance
         :type hass_url: str
@@ -92,6 +92,7 @@ class retrieve_hass:
         """
         self.logger.info("Retrieve hass get data method initiated...")
         self.df_final = pd.DataFrame()
+        x = 0 #iterate based on days
         # Looping on each day from days list
         for day in days_list:
         
@@ -115,8 +116,14 @@ class retrieve_hass:
                 try:
                     response = get(url, headers=headers)
                 except Exception:
-                    return "Request Get Error"
+                    self.logger.error("Unable to access Home Assistance instance, check URL")
+                    self.logger.error("If using addon, try setting url and token to 'empty'")
+                    return False
                 else:
+                    if response.status_code == 401:
+                        self.logger.error("Unable to access Home Assistance instance, TOKEN/KEY")
+                        self.logger.error("If using addon, try setting url and token to 'empty'")
+                        return False
                     if response.status_code > 299:
                         return f"Request Get Error: {response.status_code}"
                 '''import bz2 # Uncomment to save a serialized data for tests
@@ -126,13 +133,18 @@ class retrieve_hass:
                 try: # Sometimes when there are connection problems we need to catch empty retrieved json
                     data = response.json()[0]
                 except IndexError:
-                    self.logger.error("The retrieved JSON is empty, check that correct day or variable names are passed")
-                    self.logger.error("Either the names of the passed variables are not correct or days_to_retrieve is larger than the recorded history of your sensor (check your recorder settings)")
-                    break
+                    if x is 0:
+                        self.logger.error("The retrieved JSON is empty, A sensor:" + var + " may have 0 days of history or passed sensor may not be correct")
+                    else:
+                        self.logger.error("The retrieved JSON is empty, days_to_retrieve may be larger than the recorded history of sensor:" + var + "  (check your recorder settings)")
+                    return False
                 df_raw = pd.DataFrame.from_dict(data)
                 if len(df_raw) == 0:
-                    self.logger.error("Retrieved empty Dataframe, check that correct day or variable names are passed")
-                    self.logger.error("Either the names of the passed variables are not correct or days_to_retrieve is larger than the recorded history of your sensor (check your recorder settings)")
+                    if x is 0:
+                        self.logger.error("The retrieved Dataframe is empty, A sensor:" + var + " may have 0 days of history or passed sensor may not be correct")
+                    else:
+                        self.logger.error("Retrieved empty Dataframe, days_to_retrieve may be larger than the recorded history of sensor:" + var + "  (check your recorder settings)")
+                    return False
                 if i == 0: # Defining the DataFrame container
                     from_date = pd.to_datetime(df_raw['last_changed'], format="ISO8601").min()
                     to_date = pd.to_datetime(df_raw['last_changed'], format="ISO8601").max()
@@ -147,11 +159,13 @@ class retrieve_hass:
                 df_tp = df_tp.resample(self.freq).mean()
                 df_day = pd.concat([df_day, df_tp], axis=1)
             
+            x += 1
             self.df_final = pd.concat([self.df_final, df_day], axis=0)
         self.df_final = set_df_index_freq(self.df_final)
         if self.df_final.index.freq != self.freq:
             self.logger.error("The inferred freq from data is not equal to the defined freq in passed parameters")
-
+            return False
+        return True
     
     def prepare_data(self, var_load: str, load_negative: Optional[bool] = False, set_zero_min: Optional[bool] = True,
                      var_replace_zero: Optional[list] = None, var_interp: Optional[list] = None) -> None:
@@ -185,6 +199,10 @@ class retrieve_hass:
             self.df_final.drop([var_load], inplace=True, axis=1)
         except KeyError:
             self.logger.error("Variable "+var_load+" was not found. This is typically because no data could be retrieved from Home Assistant")
+            return False
+        except ValueError:
+            self.logger.error("sensor.power_photovoltaics and sensor.power_load_no_var_loads should not be the same")
+            return False   
         if set_zero_min: # Apply minimum values
             self.df_final.clip(lower=0.0, inplace=True, axis=1)
             self.df_final.replace(to_replace=0.0, value=np.nan, inplace=True)
@@ -215,6 +233,7 @@ class retrieve_hass:
             self.df_final.index = self.df_final.index.tz_convert(self.time_zone)
         # Drop datetimeindex duplicates on final DF
         self.df_final = self.df_final[~self.df_final.index.duplicated(keep='first')]
+        return True
     
     @staticmethod
     def get_attr_data_dict(data_df: pd.DataFrame, idx: int, entity_id: str, 
@@ -287,25 +306,25 @@ class retrieve_hass:
         else:
             state = np.round(data_df.loc[data_df.index[idx]],2)
         if type_var == 'power':
-            data = retrieve_hass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
+            data = RetrieveHass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
                                                     friendly_name, "forecasts", state)
         elif type_var == 'deferrable':
-            data = retrieve_hass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
+            data = RetrieveHass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
                                                     friendly_name, "deferrables_schedule", state)
         elif type_var == 'batt':
-            data = retrieve_hass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
+            data = RetrieveHass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
                                                     friendly_name, "battery_scheduled_power", state)
         elif type_var == 'SOC':
-            data = retrieve_hass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
+            data = RetrieveHass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
                                                     friendly_name, "battery_scheduled_soc", state)
         elif type_var == 'unit_load_cost':
-            data = retrieve_hass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
+            data = RetrieveHass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
                                                     friendly_name, "unit_load_cost_forecasts", state)
         elif type_var == 'unit_prod_price':
-            data = retrieve_hass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
+            data = RetrieveHass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
                                                     friendly_name, "unit_prod_price_forecasts", state)
         elif type_var == 'mlforecaster':
-            data = retrieve_hass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
+            data = RetrieveHass.get_attr_data_dict(data_df, idx, entity_id, unit_of_measurement, 
                                                     friendly_name, "scheduled_forecast", state)
         elif type_var == 'optim_status':
             data = {
